#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform sampler2D u_texture;
uniform float u_blurRadius;
uniform vec2 u_textureSize;
uniform sampler2D u_maskTexture;   // マスク用テクスチャ
uniform int u_colorType;
uniform vec3 u_spriteColor;

int getBlurPower(float x) {
    if(x <= 0.45){
        return 0;
    }
    else if(x <= 0.46){
        return 1;
    }
    else if(x <= 0.47){
        return 2;
    }
    else if(x <= 0.48){
        return 4;
    }
    else if(x <= 0.49){
        return 6;
    }
    else{
        return 8;
    }
}

void main() {
    vec2 texCoord = v_texCoord;
    vec4 originalColor = texture2D(u_texture, texCoord);

    vec2 center = vec2(0.5, 0.5);
    float distanceFromCenter = distance(texCoord, center);
    float maxDistance = distance(vec2(0.0, 0.0), center);

    float normalizedDistance = distanceFromCenter / maxDistance;
    
    vec4 maskColor = texture2D(u_maskTexture, texCoord);
    float maskAlpha = maskColor.a; // マスク用テクスチャのアルファチャンネルを使用
    if(maskColor.a == 0.0){
        discard;
    }
    else if(maskColor.a == 1.0){
        vec4 finalColor = originalColor;
        //死んだときの赤だけは元スプライトからうまくとれない。。。
        if(u_colorType == 2){
            finalColor = vec4(originalColor.r, 0.0, 0.0, originalColor.a);
        }
        gl_FragColor = vec4(finalColor.rgb * u_spriteColor.rgb, finalColor.a);
    }
    else{
        float blurAmount = normalizedDistance * u_blurRadius;

        vec4 sum = vec4(0.0);
        vec2 texOffset = vec2(blurAmount / u_textureSize.x, blurAmount / u_textureSize.y);

        int power = getBlurPower(distanceFromCenter);
        for (int x = -1 * power; x <= power; x++) {
            for (int y = -1 * power; y <= power; y++) {
                vec2 offset = vec2(float(x) * texOffset.x, float(y) * texOffset.y);
                sum += texture2D(u_texture, texCoord + offset);
            }
        }

        sum /= float((power + power + 1) * (power + power + 1));
        vec4 baseColor = mix(originalColor, sum, normalizedDistance) * v_fragmentColor;
        vec4 finalColor = vec4(baseColor.rgb * maskAlpha * u_spriteColor.rgb, baseColor.a * maskAlpha);
        //死んだときの赤だけは元スプライトからうまくとれない。。。
        if(u_colorType == 2){
            finalColor = vec4(finalColor.r, 0.0, 0.0, finalColor.a);
        }
        gl_FragColor = finalColor;
    }
}
